#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) nexB Inc. and others. All rights reserved.
# ScanCode is a trademark of nexB Inc.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/aboutcode-org/skeleton for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#
import argparse
import pathlib

import utils_requirements

"""
Utilities to manage requirements files.
NOTE: this should use ONLY the standard library and not import anything else
because this is used for boostrapping with no requirements installed.
"""


def gen_dev_requirements():
    description = """
    Create or overwrite the `--dev-requirements-file` pip requirements FILE with
    all Python packages found installed in `--site-packages-dir`. Exclude
    package names also listed in the --main-requirements-file pip requirements
    FILE (that are assume to the production requirements and therefore to always
    be present in addition to the development requirements).
    """
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument(
        "-s",
        "--site-packages-dir",
        type=pathlib.Path,
        required=True,
        metavar="DIR",
        help='Path to the "site-packages" directory where wheels are installed such as lib/python3.6/site-packages',
    )
    parser.add_argument(
        "-d",
        "--dev-requirements-file",
        type=pathlib.Path,
        metavar="FILE",
        default="requirements-dev.txt",
        help="Path to the dev requirements file to update or create.",
    )
    parser.add_argument(
        "-r",
        "--main-requirements-file",
        type=pathlib.Path,
        default="requirements.txt",
        metavar="FILE",
        help="Path to the main requirements file. Its requirements will be excluded "
        "from the generated dev requirements.",
    )
    args = parser.parse_args()

    utils_requirements.lock_dev_requirements(
        dev_requirements_file=args.dev_requirements_file,
        main_requirements_file=args.main_requirements_file,
        site_packages_dir=args.site_packages_dir,
    )


if __name__ == "__main__":
    gen_dev_requirements()



# -*- coding: utf-8 -*-
#
# SPDX-License-Identifier: LicenseRef-scancode-public-domain
# See https://github.com/aboutcode-org/license-expression for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.

"""
Tests for Aho-Corasick string search algorithm.
Original Author: Wojciech Muła, wojciech_mula@poczta.onet.pl
WWW            : http://0x80.pl
License        : public domain

Modified for use in the license_expression library.
"""

import unittest

from license_expression._pyahocorasick import Trie
from license_expression._pyahocorasick import Token


class TestTrie(unittest.TestCase):

    def test_add_can_get(self):
        t = Trie()
        t.add('python', 'value')
        assert ('python', 'value') == t.get('python')

    def test_add_existing_WordShouldReplaceAssociatedValue(self):
        t = Trie()
        t.add('python', 'value')
        assert ('python', 'value') == t.get('python')

        t.add('python', 'other')
        assert ('python', 'other') == t.get('python')

    def test_get_UnknowWordWithoutDefaultValueShouldRaiseException(self):
        t = Trie()
        with self.assertRaises(KeyError):
            t.get('python')

    def test_get_UnknowWordWithDefaultValueShouldReturnDefault(self):
        t = Trie()
        self.assertEqual(t.get('python', 'default'), 'default')

    def test_exists_ShouldDetectAddedWords(self):
        t = Trie()
        t.add('python', 'value')
        t.add('ada', 'value')

        self.assertTrue(t.exists('python'))
        self.assertTrue(t.exists('ada'))

    def test_exists_ShouldReturnFailOnUnknownWord(self):
        t = Trie()
        t.add('python', 'value')

        self.assertFalse(t.exists('ada'))

    def test_is_prefix_ShouldDetecAllPrefixesIncludingWord(self):
        t = Trie()
        t.add('python', 'value')
        t.add('ada lovelace', 'value')

        self.assertFalse(t.is_prefix('a'))
        self.assertFalse(t.is_prefix('ad'))
        self.assertTrue(t.is_prefix('ada'))

        self.assertFalse(t.is_prefix('p'))
        self.assertFalse(t.is_prefix('py'))
        self.assertFalse(t.is_prefix('pyt'))
        self.assertFalse(t.is_prefix('pyth'))
        self.assertFalse(t.is_prefix('pytho'))
        self.assertTrue(t.is_prefix('python'))

        self.assertFalse(t.is_prefix('lovelace'))

    def test_items_ShouldReturnAllItemsAlreadyAddedToTheTrie(self):
        t = Trie()

        t.add('python', 1)
        t.add('ada', 2)
        t.add('perl', 3)
        t.add('pascal', 4)
        t.add('php', 5)
        t.add('php that', 6)

        result = list(t.items())
        self.assertIn(('python', 1), result)
        self.assertIn(('ada', 2), result)
        self.assertIn(('perl', 3), result)
        self.assertIn(('pascal', 4), result)
        self.assertIn(('php', 5), result)
        self.assertIn(('php that', 6), result)

    def test_keys_ShouldReturnAllKeysAlreadyAddedToTheTrie(self):
        t = Trie()

        t.add('python', 1)
        t.add('ada', 2)
        t.add('perl', 3)
        t.add('pascal', 4)
        t.add('php', 5)
        t.add('php that', 6)

        result = list(t.keys())
        self.assertIn('python', result)
        self.assertIn('ada', result)
        self.assertIn('perl', result)
        self.assertIn('pascal', result)
        self.assertIn('php', result)
        self.assertIn('php that', result)

    def test_values_ShouldReturnAllValuesAlreadyAddedToTheTrie(self):
        t = Trie()

        t.add('python', 1)
        t.add('ada', 2)
        t.add('perl', 3)
        t.add('pascal', 4)
        t.add('php', 5)

        result = list(t.values())
        self.assertIn(1, result)
        self.assertIn(2, result)
        self.assertIn(3, result)
        self.assertIn(4, result)
        self.assertIn(5, result)

    def test_iter_should_not_return_non_matches_by_default(self):

        def get_test_automaton():
            words = 'he her hers his she hi him man himan'.split()
            t = Trie()
            for w in words:
                t.add(w, w)
            t.make_automaton()
            return t

        test_string = 'he she himan'

        t = get_test_automaton()
        result = list(t.iter(test_string))
        assert 'he she himan'.split() == [r.value for r in result]

    def test_iter_should_can_return_non_matches_optionally(self):

        def get_test_automaton():
            words = 'he her hers his she hi him man himan'.split()
            t = Trie()
            for w in words:
                t.add(w, w)
            t.make_automaton()
            return t

        test_string = '  he she junk  himan  other stuffs   '
        #                        111111111122222222223333333
        #              0123456789012345678901234567890123456

        t = get_test_automaton()
        result = list(
            t.iter(test_string, include_unmatched=True, include_space=True))
        expected = [
            Token(0, 1, u'  ', None),
            Token(2, 3, u'he', u'he'),
            Token(4, 4, u' ', None),
            Token(5, 7, u'she', u'she'),
            Token(8, 8, u' ', None),
            Token(9, 12, u'junk', None),
            Token(13, 14, u'  ', None),
            Token(15, 19, u'himan', u'himan'),
            Token(20, 21, u'  ', None),
            Token(22, 26, u'other', None),
            Token(27, 27, u' ', None),
            Token(28, 33, u'stuffs', None),
            Token(34, 36, u'   ', None),
        ]

        assert expected == result

    def test_iter_vs_tokenize(self):

        def get_test_automaton():
            words = '( AND ) OR'.split()
            t = Trie()
            for w in words:
                t.add(w, w)
            t.make_automaton()
            return t

        test_string = '((l-a + AND l-b) OR (l -c+))'

        t = get_test_automaton()
        result = list(
            t.iter(test_string, include_unmatched=True, include_space=True))
        expected = [
            Token(0, 0, u'(', u'('),
            Token(1, 1, u'(', u'('),
            Token(2, 4, u'l-a', None),
            Token(5, 5, u' ', None),
            Token(6, 6, u'+', None),
            Token(7, 7, u' ', None),
            Token(8, 10, u'AND', u'AND'),
            Token(11, 11, u' ', None),
            Token(12, 14, u'l-b', None),
            Token(15, 15, u')', u')'),
            Token(16, 16, u' ', None),
            Token(17, 18, u'OR', u'OR'),
            Token(19, 19, u' ', None),
            Token(20, 20, u'(', u'('),
            Token(21, 21, u'l', None),
            Token(22, 22, u' ', None),
            Token(23, 25, u'-c+', None),
            Token(26, 26, u')', u')'),
            Token(27, 27, u')', u')')
        ]

        assert expected == result

        result = list(t.tokenize(
            test_string, include_unmatched=True, include_space=True))
        assert expected == result

    def test_tokenize_with_unmatched_and_space(self):

        def get_test_automaton():
            words = '( AND ) OR'.split()
            t = Trie()
            for w in words:
                t.add(w, w)
            t.make_automaton()
            return t

        test_string = '((l-a + AND l-b) OR an (l -c+))'
        #                        111111111122222222223
        #              0123456789012345678901234567890
        t = get_test_automaton()
        result = list(t.tokenize(
            test_string, include_unmatched=True, include_space=True))
        expected = [
            Token(0, 0, u'(', u'('),
            Token(1, 1, u'(', u'('),
            Token(2, 4, u'l-a', None),
            Token(5, 5, u' ', None),
            Token(6, 6, u'+', None),
            Token(7, 7, u' ', None),
            Token(8, 10, u'AND', u'AND'),
            Token(11, 11, u' ', None),
            Token(12, 14, u'l-b', None),
            Token(15, 15, u')', u')'),
            Token(16, 16, u' ', None),
            Token(17, 18, u'OR', u'OR'),
            Token(19, 19, u' ', None),
            Token(20, 21, u'an', None),
            Token(22, 22, u' ', None),
            Token(23, 23, u'(', u'('),
            Token(24, 24, u'l', None),
            Token(25, 25, u' ', None),
            Token(26, 28, u'-c+', None),
            Token(29, 29, u')', u')'),
            Token(30, 30, u')', u')')
        ]

        assert expected == result
        assert test_string == ''.join(t.string for t in result)

    def test_iter_with_unmatched_simple(self):
        t = Trie()
        t.add('And', 'And')
        t.make_automaton()
        test_string = 'AND  an a And'
        result = list(t.iter(test_string))
        assert ['And', 'And'] == [r.value for r in result]

    def test_iter_with_unmatched_simple2(self):
        t = Trie()
        t.add('AND', 'AND')
        t.make_automaton()
        test_string = 'AND  an a and'
        result = list(t.iter(test_string))
        assert ['AND', 'AND'] == [r.value for r in result]

    def test_iter_with_unmatched_simple3(self):
        t = Trie()
        t.add('AND', 'AND')
        t.make_automaton()
        test_string = 'AND  an a andersom'
        result = list(t.iter(test_string))
        assert ['AND'] == [r.value for r in result]

    def test_iter_simple(self):
        t = Trie()
        t.add('AND', 'AND')
        t.add('OR', 'OR')
        t.add('WITH', 'WITH')
        t.add('(', '(')
        t.add(')', ')')
        t.add('GPL-2.0', 'GPL-2.0')
        t.add('mit', 'MIT')
        t.add('Classpath', 'Classpath')
        t.make_automaton()
        test_string = '(GPL-2.0 with Classpath) or (gpl-2.0) and (classpath or  gpl-2.0 OR mit) '
        #                        111111111122222222223333333333444444444455555555556666666666777
        #              0123456789012345678901234567890123456789012345678901234567890123456789012
        result = list(t.iter(test_string))
        expected = [
            Token(0, 0, u'(', u'('),
            Token(1, 7, u'GPL-2.0', u'GPL-2.0'),
            Token(9, 12, u'with', u'WITH'),
            Token(14, 22, u'Classpath', u'Classpath'),
            Token(23, 23, u')', u')'),
            Token(25, 26, u'or', u'OR'),
            Token(28, 28, u'(', u'('),
            Token(29, 35, u'gpl-2.0', u'GPL-2.0'),
            Token(36, 36, u')', u')'),
            Token(38, 40, u'and', u'AND'),
            Token(42, 42, u'(', u'('),
            Token(43, 51, u'classpath', u'Classpath'),
            Token(53, 54, u'or', u'OR'),
            Token(57, 63, u'gpl-2.0', u'GPL-2.0'),
            Token(65, 66, u'OR', u'OR'),
            Token(68, 70, u'mit', u'MIT'),
            Token(71, 71, u')', u')')
        ]

        assert expected == result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) nexB Inc. and others. All rights reserved.
# ScanCode is a trademark of nexB Inc.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/aboutcode-org/skeleton for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#
import io
import os
import zipfile

import requests
import saneyaml

from packvers import version as packaging_version

"""
Utility to create and retrieve package and ABOUT file data from DejaCode.
"""

DEJACODE_API_KEY = os.environ.get("DEJACODE_API_KEY", "")
DEJACODE_API_URL = os.environ.get("DEJACODE_API_URL", "")

DEJACODE_API_URL_PACKAGES = f"{DEJACODE_API_URL}packages/"
DEJACODE_API_HEADERS = {
    "Authorization": "Token {}".format(DEJACODE_API_KEY),
    "Accept": "application/json; indent=4",
}


def can_do_api_calls():
    if not DEJACODE_API_KEY and DEJACODE_API_URL:
        print(
            "DejaCode DEJACODE_API_KEY and DEJACODE_API_URL not configured. Doing nothing")
        return False
    else:
        return True


def fetch_dejacode_packages(params):
    """
    Return a list of package data mappings calling the package API with using
    `params` or an empty list.
    """
    if not can_do_api_calls():
        return []

    response = requests.get(
        DEJACODE_API_URL_PACKAGES,
        params=params,
        headers=DEJACODE_API_HEADERS,
    )

    return response.json()["results"]


def get_package_data(distribution):
    """
    Return a mapping of package data or None for a Distribution `distribution`.
    """
    results = fetch_dejacode_packages(distribution.identifiers())

    len_results = len(results)

    if len_results == 1:
        return results[0]

    elif len_results > 1:
        print(
            f"More than 1 entry exists, review at: {DEJACODE_API_URL_PACKAGES}")
    else:
        print("Could not find package:", distribution.download_url)


def update_with_dejacode_data(distribution):
    """
    Update the Distribution `distribution` with DejaCode package data. Return
    True if data was updated.
    """
    package_data = get_package_data(distribution)
    if package_data:
        return distribution.update(package_data, keep_extra=False)

    print(f"No package found for: {distribution}")


def update_with_dejacode_about_data(distribution):
    """
    Update the Distribution `distribution` wiht ABOUT code data fetched from
    DejaCode. Return True if data was updated.
    """
    package_data = get_package_data(distribution)
    if package_data:
        package_api_url = package_data["api_url"]
        about_url = f"{package_api_url}about"
        response = requests.get(about_url, headers=DEJACODE_API_HEADERS)
        # note that this is YAML-formatted
        about_text = response.json()["about_data"]
        about_data = saneyaml.load(about_text)

        return distribution.update(about_data, keep_extra=True)

    print(f"No package found for: {distribution}")


def fetch_and_save_about_files(distribution, dest_dir="thirdparty"):
    """
    Fetch and save in `dest_dir` the .ABOUT, .LICENSE and .NOTICE files fetched
    from DejaCode for a Distribution `distribution`. Return True if files were
    fetched.
    """
    package_data = get_package_data(distribution)
    if package_data:
        package_api_url = package_data["api_url"]
        about_url = f"{package_api_url}about_files"
        response = requests.get(about_url, headers=DEJACODE_API_HEADERS)
        about_zip = response.content
        with io.BytesIO(about_zip) as zf:
            with zipfile.ZipFile(zf) as zi:
                zi.extractall(path=dest_dir)
        return True

    print(f"No package found for: {distribution}")


def find_latest_dejacode_package(distribution):
    """
    Return a mapping of package data for the closest version to
    a Distribution `distribution` or None.
    Return the newest of the packages if prefer_newest is True.
    Filter out version-specific attributes.
    """
    ids = distribution.purl_identifiers(skinny=True)
    packages = fetch_dejacode_packages(params=ids)
    if not packages:
        return

    for package_data in packages:
        matched = (
            package_data["download_url"] == distribution.download_url
            and package_data["version"] == distribution.version
            and package_data["filename"] == distribution.filename
        )

        if matched:
            return package_data

    # there was no exact match, find the latest version
    # TODO: consider the closest version rather than the latest
    # or the version that has the best data
    with_versions = [(packaging_version.parse(p["version"]), p)
                     for p in packages]
    with_versions = sorted(with_versions)
    latest_version, latest_package_version = sorted(with_versions)[-1]
    print(
        f"Found DejaCode latest version: {latest_version} " f"for dist: {distribution.package_url}",
    )

    return latest_package_version


def create_dejacode_package(distribution):
    """
    Create a new DejaCode Package a Distribution `distribution`.
    Return the new or existing package data.
    """
    if not can_do_api_calls():
        return

    existing_package_data = get_package_data(distribution)
    if existing_package_data:
        return existing_package_data

    print(f"Creating new DejaCode package for: {distribution}")

    new_package_payload = {
        # Trigger data collection, scan, and purl
        "collect_data": 1,
    }

    fields_to_carry_over = [
        "download_url" "type",
        "namespace",
        "name",
        "version",
        "qualifiers",
        "subpath",
        "license_expression",
        "copyright",
        "description",
        "homepage_url",
        "primary_language",
        "notice_text",
    ]

    for field in fields_to_carry_over:
        value = getattr(distribution, field, None)
        if value:
            new_package_payload[field] = value

    response = requests.post(
        DEJACODE_API_URL_PACKAGES,
        data=new_package_payload,
        headers=DEJACODE_API_HEADERS,
    )
    new_package_data = response.json()
    if response.status_code != 201:
        raise Exception(f"Error, cannot create package for: {distribution}")

    print(f'New Package created at: {new_package_data["absolute_url"]}')
    return new_package_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) nexB Inc. and others. All rights reserved.
# ScanCode is a trademark of nexB Inc.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.
# See https://github.com/aboutcode-org/skeleton for support or download.
# See https://aboutcode.org for more information about nexB OSS projects.
#
import argparse
import pathlib

import utils_requirements

"""
Utilities to manage requirements files.
NOTE: this should use ONLY the standard library and not import anything else
because this is used for boostrapping with no requirements installed.
"""


def gen_requirements():
    description = """
    Create or replace the `--requirements-file` file FILE requirements file with all
    locally installed Python packages.all Python packages found installed in `--site-packages-dir`
    """
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument(
        "-s",
        "--site-packages-dir",
        dest="site_packages_dir",
        type=pathlib.Path,
        required=True,
        metavar="DIR",
        help="Path to the 'site-packages' directory where wheels are installed such as lib/python3.6/site-packages",
    )
    parser.add_argument(
        "-r",
        "--requirements-file",
        type=pathlib.Path,
        metavar="FILE",
        default="requirements.txt",
        help="Path to the requirements file to update or create.",
    )

    args = parser.parse_args()

    utils_requirements.lock_requirements(
        site_packages_dir=args.site_packages_dir,
        requirements_file=args.requirements_file,
    )


if __name__ == "__main__":
    gen_requirements()
